#include <HT32F125x.h>                    // 

#include "HD61202_lowlevel.h"
#include "fonts.h"
#include "font5x7s.h"

/* Low level functions */

void LCD_SelSide(unsigned char side)
{
	if(side)	{
		LCD_CS2=1;		// Rise CS2
		LCD_CS1=0;		// Lower CS1
	}
	else	{
		LCD_CS1=1;		// Rise CS1
		LCD_CS2=0;		// Lower CS2
	}
}

void LCD_CalcPage(unsigned char y,unsigned char *page,unsigned char *row)
{
	*page=(y>>3);			// page = integer part of y coord /8
	*row=y-((*page)<<3);		// row = remainder
}
	

unsigned char LCD_ReadData()
{
	LCD_Status();		// ID returns low
	LCD_ID=1;		// Rise ID (Data)
	return(LCD_Read());
}

void LCD_WriteCmd(unsigned char cmd)
{
	LCD_Status();		// ID returns low
	LCD_Write(cmd);
}

void LCD_WriteData(unsigned char dat)
{
	LCD_Status();
	LCD_ID=1;		// Rise ID (Data)
	LCD_Write(dat);
}

void LCD_init ()
{
	LCD_setupI0();
	//MsDelay ( 1000 );				// wait for LCD to reset itself

	LCD_SelSide(1);
	LCD_WriteCmd (0x3F);			// Display on
	LCD_SelSide(0);
	LCD_WriteCmd (0x3F);			// Display on
}

/* High level functions */

void LCD_home ( void )
{
	LCD_SelSide(1);
	LCD_WriteCmd (0x40);			// address 0
	LCD_WriteCmd (0xC0);			// display starts at address 0
	LCD_SelSide(0);
	LCD_WriteCmd (0x40);			// address 0
	LCD_WriteCmd (0xC0);			// display starts at address 0
}

void LCD_fill(unsigned char pattern)
{
unsigned char i,page;
	LCD_home();										// address 0,0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
	}
}

#define LCD_clear() LCD_fill(0)


void LCD_plot (unsigned char x, unsigned char y)
{
unsigned char page,row,dat;
	LCD_SelSide(x>63);				// 0-63 => side 0; 63>127 => side 1
	x=(x>63)?(x-64):x;				// correct x address if >63
	LCD_WriteCmd (0x40+x);				// address = x or x-64
	LCD_CalcPage(y,&page,&row);			// calculate page and row within page
	LCD_WriteCmd (0xB8+page);			// set page address
	LCD_ReadData();					// dummy read after address setting
	dat=LCD_ReadData();				// get data at that address
	dat|=(1<<row);					// set requested bit
	LCD_WriteCmd (0x40+x);				// set address again (reading increments counter)
	LCD_WriteData(dat);
}

void LCD_dump (uint8_t *imgdata)
{
unsigned char i,page;

	LCD_home();										// address 0,0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(*(imgdata++));		// send left half for each page
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address	
		for(i=0;i<64;i++)
			LCD_WriteData(*(imgdata++));		// send right half for each page
	}
}

void LCD_putchr ( char chr )
{
unsigned char i;
	for(i=0;i<5;i++)
		LCD_WriteData(font5x7s[5*(chr-0x20)+i]);	// send character
	LCD_WriteData(0);										// send intercharacter separator	
}

void LCD_printat (unsigned char row, unsigned char col, char *ptr)
{
unsigned char x;
	do {
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr (*ptr++);
		col++;
	} 	while (*ptr);
}

void LCD_icon (uint8_t *imgdata,unsigned char x, unsigned char page,unsigned char width, unsigned char height)
{
unsigned char bytes,row;

	while(height--) {                       // para cada fila
	  bytes=width;                          // bytes a enviar por fila
	  row=x;                                // posicion de inicio en la fila
	  LCD_SelSide(x>63);			// 0-63 => side 0; 63>127 => side 1
	  x=(x>63)?(x-64):x;			// corrige x si x >63
	  LCD_WriteCmd (0x40+x);		// address = x o x-64
	  LCD_WriteCmd (0xB8+page);		// page address	
	  while(bytes--) {
		LCD_WriteData(*(imgdata++));	// datos
		if(++row==64) {                 // cambio de controlador ?
		  row=0;                        // corrige columna
		  LCD_SelSide(1);		// controlador derecho
		  LCD_WriteCmd (0x40);		// address = 0
		  LCD_WriteCmd (0xB8+page);	// page address
		}
	  }
	  page++;
	}
}

void LCD_area (unsigned char pattern,unsigned char x, unsigned char page,unsigned char width, unsigned char height)
{
unsigned char bytes,row;

	while(height--) {                       // para cada fila
	  bytes=width;                          // bytes a enviar por fila
	  row=x;                                // posicion de inicio en la fila
	  LCD_SelSide(x>63);			// 0-63 => side 0; 63>127 => side 1
	  x=(x>63)?(x-64):x;			// corrige x si x >63
	  LCD_WriteCmd (0x40+x);		// address = x o x-64
	  LCD_WriteCmd (0xB8+page);		// page address	
	  while(bytes--) {
		LCD_WriteData(pattern);	        // datos
		if(++row==64) {                 // cambio de controlador ?
		  row=0;                        // corrige columna
		  LCD_SelSide(1);		// controlador derecho
		  LCD_WriteCmd (0x40);		// address = 0
		  LCD_WriteCmd (0xB8+page);	// page address
		}
	  }
	  page++;
	}
}

void LCD_invert (unsigned char x, unsigned char page,unsigned char width)
{
unsigned char bytes,row,dat;

	  bytes=width;                          // bytes a enviar por fila
	  LCD_SelSide(x>63);			// 0-63 => side 0; 63>127 => side 1
	  x=(x>63)?(x-64):x;			// corrige x si x >63
	  LCD_WriteCmd (0x40+x);		// address = x o x-64
	  LCD_WriteCmd (0xB8+page);		// page address	
	  row=x;                                // posicion de inicio en la fila
	  while(bytes--) {
	    LCD_ReadData();					// dummy read after address setting
	    dat=LCD_ReadData();				// get data at that address
	    dat^=0xFF;                          // invert
	    LCD_WriteCmd (0x40+row);				// set address again (reading increments counter)
	    LCD_WriteData(dat);	        // datos
	    if(++row==64) {                 // cambio de controlador ?
	      row=0;                        // corrige columna
	      LCD_SelSide(1);		// controlador derecho
	      LCD_WriteCmd (0x40);		// address = 0
	      LCD_WriteCmd (0xB8+page);	// page address
	    }
	  }
}


// Bresenham Line Algorithm

void LCD_line(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2)
{
unsigned char x,y;
signed char dx,dy,incx,incy,balance;

        if (x2 >= x1)
        {
                dx = x2 - x1;
                incx = 1;
        }
        else
        {
                dx = x1 - x2;
                incx = -1;
        }

        if (y2 >= y1)
        {
                dy = y2 - y1;
                incy = 1;
        }
        else
        {
                dy = y1 - y2;
                incy = -1;
        }

        x = x1;
        y = y1;

        if (dx >= dy)
        {
                dy <<= 1;
                balance = dy - dx;
                dx <<= 1;

                while (x != x2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                y += incy;
                                balance -= dx;
                        }
                        balance += dy;
                        x += incx;
                } LCD_plot(x, y);
        }
        else
        {
                dx <<= 1;
                balance = dx - dy;
                dy <<= 1;

                while (y != y2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                x += incx;
                                balance -= dy;
                        }
                        balance += dx;
                        y += incy;
                } LCD_plot(x, y);
        }
}

void LCD_rectangle(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2)
{
	LCD_line(x1,y1,x2,y1);
	LCD_line(x2,y1,x2,y2);
	LCD_line(x2,y2,x1,y2);
	LCD_line(x1,y2,x1,y1);
}





void LCD_putchr6 ( unsigned char chr )
{
unsigned char i;
	for(i=0;i<5;i++)
		LCD_WriteData(font5x7[5*chr+i]);				// send character
	LCD_WriteData(0);										// send intercharacter separator	
}



void LCD_putchr8 ( unsigned char chr )
{
unsigned char i;
	for(i=0;i<8;i++)
		LCD_WriteData(font8x8[(chr<<3)+i]);				// send character
}

void LCD_putchrB ( unsigned char chr , unsigned char row, unsigned char col)
{
unsigned char i;

	for(i=0;i<8;i++)
		LCD_WriteData(font8x15[(chr<<4)+i]);			// send upper part
	LCD_WriteCmd (0xB9+row);							// point to next page address
	LCD_WriteCmd (0x40+col);							// restore original column
	for(;i<16;i++)
		LCD_WriteData(font8x15[(chr<<4)+i]);			// send lower part
}



void LCD_print6at (unsigned char row, unsigned char col, char *ptr)
{
unsigned char x;
	do {
		if(col>19){
			col=0;
			row++;
		}
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr6 (*ptr++);
		col++;
	} 	while (*ptr);
}

void LCD_print8at (unsigned char row, unsigned char col, char format, char *ptr)
{
unsigned char x;
	do {
		if(col>15){
			col=0;
			row++;
			if(format=='B')
				row++;
		}
		x=(col>7)?8*(col-8):8*col;					// correct x address if col>9
		if(col>7)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address
		switch(format){
			case 'B':
				LCD_putchrB (*ptr++,row,x);
				break;
			default:
				LCD_putchr8 (*ptr++);
				break;
		}
		col++;
	} 	while (*ptr);
}
