#include <HT32F125x.h>

#include "HD61202_lowlevel.h"

/* 72 MHz -> 14ns, while loop is ~3 instructions -> ~100ns
parameter + call + ret is ~5 instructions -> ~150ns extra
this does not take into account the AHB+APB internal clocking and estimates for flash 2 wait-states */
void DILEI(int x){volatile int mydelay;mydelay=(x);while(mydelay--);}

/* Low level functions */

void LCD_Write(unsigned char dat)
{
  /* HD61202U: 150ns address setup, 200ns data setup, 450ns E width */
    HT_GPIOA->DOUTR = (dat<<8);			// write data
	if(dat&(1<<2))
		HT_GPIOB->DOUTR |= (1<<15);
	else
		HT_GPIOB->DOUTR &= ~(1<<15);
	HT_GPIOA->DIRCR |= 0xFB00;       // PA.8,9,11-15 defined as Outputs, put data in bus
	HT_GPIOB->DIRCR |= (1<<15);      // PB-15 defined as Output, put data in bus
	LCD_RW=0;	// Lower RW (Write) ? ns
	       // need  200 - ? - ?? ns delay
		   DILEI(1);
	LCD_E=1;	        // Rise E 	?? ns
	       // need  450 - ?? ns delay
		   DILEI(3);
	LCD_E=0;	// Lower E ?? ns
	       // need  450 - RET ns delay
		   DILEI(2);
}

unsigned char LCD_Read()
{
  /* HD61202U: 150ns address setup, 450ns E width, 320ns data delay */
unsigned char dat;
	LCD_RW=1;		// Rise RW (Read)
	HT_GPIOA->DIRCR &= ~0xFB00;             // PA.8,9,11-15 defined as Inputs, remove data from bus
	HT_GPIOB->DIRCR &= ~(1<<15);             // PB15 defined as Input, remove data from bus
	       // need  150 - ?? ns delay
		   DILEI(1);
	LCD_E=1;	        // Rise E 	?? ns
	       // need  320 - ? ns delay
		   DILEI(2);
	dat=(unsigned char)(HT_GPIOA->DINR >> 8);	           // read data, ? ns
	if(HT_GPIOB->DINR & (1<<15))				// map PB15 to D2, ? ns
		dat |= (1<<2);
	else
		dat &= ~(1<<2);
	       // need  450 - 320 - ?? - ? ns delay
		   DILEI(1);
	LCD_E=0;	// Lower E ?? ns
	       // need  450 - return() ns delay
		   DILEI(2);
	return(dat);
}

unsigned char LCD_Status()
{
unsigned char status;
    LCD_ID=0;       	     // Lower ID (Cmd)
	do {
		status=LCD_Read();   	// read status (busy flag)
	} while(status&(1<<LCD_BUSY));   // loop while busy=1
	return(status);
}
void LCD_setupI0 ()
{
	/* Setup I/O */
	HT_CKCU->APBCCR0 |= (1<<17)+(1<<16)+(1<<14);// 17=PBEN, 16=PAEN, 14=AFIOEN, enable APB clocks for GPIOA, GPIOB, and AFIO
	HT_AFIO->GPACFGR = 0x54000000;				// PA15,14,13 -> AF1 (lose SWD, no debugging)
	HT_AFIO->GPBCFGR = 0x00100000;				// PB10 -> AF1
	HT_GPIOA->INER &= ~0x0400;                      // PA.10 input circuit disabled
	HT_GPIOA->INER |= 0xFB00;                      // PA.8,9,11-15 input circuits enabled
	HT_GPIOB->INER |= 0x8000;                      // PB.15 input circuit enabled
	HT_GPIOA->DIRCR |= 0xFF00;                      // PA.8-15 defined as Outputs
	HT_GPIOB->DIRCR |= 0xFC00;                     // PB.10,11,12,13,14,15 defined as Outputs
	LCD_E=0;
}
